@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2022 DomainTools LLC
  * Copyright (c) 2012-2018 by Farsight Security, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
@@ -143,7 +144,6 @@ mtbl_reader_init_fd(int fd, const struct mtbl_reader_options *opt)
 	size_t metadata_offset;
 
 	size_t index_len, index_len_len;
-	uint32_t index_crc;
 	uint8_t *index_data;
 
 	int ret = fstat(fd, &ss);
@@ -169,14 +169,20 @@ mtbl_reader_init_fd(int fd, const struct mtbl_reader_options *opt)
 	}
 
 	/**
-	 * Sanitize the index block offset.
-	 * We calculate the maximum possible index block offset for this file to
-	 * be the total size of the file (r->len_data) minus the length of the
-	 * metadata block (MTBL_METADATA_SIZE) minus the length of the minimum
-	 * sized block, which requires 4 fixed-length 32-bit integers (16 bytes).
+	 * Verify that the index block offset leaves room for the metadata block
+	 * and a block of the minimum length, which differs between the mtbl format
+	 * V1 and V2. A minimal V2 block requires three fixed-length 32-bit integers
+	 * and a varint length (1 byte) for a size of 13 bytes. V1 used a fixed-length
+	 * 32-bit integer length, making the minimum length 16.
 	 */
-	const uint64_t max_index_block_offset = r->len_data - MTBL_METADATA_SIZE - 16;
-	if (r->m.index_block_offset > max_index_block_offset) {
+	uint64_t end, min_block_length = 13;
+
+	if (r->m.file_version == MTBL_FORMAT_V1)
+		min_block_length = 16;
+
+	end = r->m.index_block_offset + MTBL_METADATA_SIZE + min_block_length;
+	if ((end > r->len_data) ||		/* offset too large */
+	    (end < r->m.index_block_offset)) {  /* offset causes overflow */
 		mtbl_reader_destroy(&r);
 		return (NULL);
 	}
@@ -195,9 +201,13 @@ mtbl_reader_init_fd(int fd, const struct mtbl_reader_options *opt)
 			return NULL;
 		}
 	}
-	index_crc = mtbl_fixed_decode32(r->data + r->m.index_block_offset + index_len_len);
 	index_data = r->data + r->m.index_block_offset + index_len_len + sizeof(uint32_t);
-	assert(index_crc == mtbl_crc32c(index_data, index_len));
+	if (r->opt.verify_checksums) {
+		uint32_t index_crc, calc_crc;
+		index_crc = mtbl_fixed_decode32(r->data + r->m.index_block_offset + index_len_len);
+		calc_crc = mtbl_crc32c(index_data, index_len);
+		assert(index_crc == calc_crc);
+	}
 	r->index = block_init(index_data, index_len, false);
 	r->source = mtbl_source_init(reader_iter,
 				     reader_get,
@@ -412,6 +422,34 @@ reader_iter_free(void *v)
 	}
 }
 
+static bool
+needs_index_seek(struct reader_iter *it, const uint8_t *seek_key, size_t len_seek_key)
+{
+	const uint8_t *key, *val;
+	size_t len_key, len_val;
+
+	if (it->first || it->b == NULL)
+		return true;
+
+	/* Check for seek before current key. */
+	if (!block_iter_get(it->bi, &key, &len_key, &val, &len_val))
+		return true;
+	if (bytes_compare(key, len_key, seek_key, len_seek_key) > 0)
+		return true;
+
+	/*
+	 * We are seeking forward of the current key. Check our
+	 * current index block position to see if the target is
+	 * after the end of the current block.
+	 */
+	if (!block_iter_get(it->index_iter, &key, &len_key, &val, &len_val))
+		return true;
+	if (bytes_compare(key, len_key, seek_key, len_seek_key) < 0)
+		return true;
+
+	return false;
+}
+
 static mtbl_res
 reader_iter_seek(void *v,
 	       const uint8_t *key, size_t len_key)
@@ -422,7 +460,8 @@ reader_iter_seek(void *v,
 	size_t len_ikey, len_ival;
 	uint64_t new_offset;
 
-	block_iter_seek(it->index_iter, key, len_key);
+	if (needs_index_seek(it, key, len_key))
+		block_iter_seek(it->index_iter, key, len_key);
 
 	if (!block_iter_get(it->index_iter, &ikey, &len_ikey, &ival, &len_ival)) {
 		/* This seek puts us after the last key, so we mark the
